#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" This package provides the capability to schedule and account for transactions for the exchange of electric power between companies. It includes transations for megawatts which are generated, consumed, lost, passed through, sold and purchased. These classes are used by Accounting and Billing for Energy, Generation Capacity, Transmission, and Ancillary Services.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61968.Common import Document
from CIM.IEC61970.Core import Curve
from CIM import Element
from CIM.IEC61970.Core import IdentifiedObject
from CIM.IEC61970.ControlArea import ControlArea
from CIM.IEC61970.Core import PowerSystemResource
from CIM.IEC61968.Common import Agreement
from CIM.IEC61970.Core import RegularIntervalSchedule
from CIM.IEC61970.Domain import ActivePower
from CIM.IEC61970.Domain import Money
from CIM.IEC61970.Domain import RealEnergy
from CIM.IEC61970.Domain import Frequency



from enthought.traits.api import Instance, List, Property, Enum, Bool, Str, Date, Int
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


AreaControlMode = Enum("OFF", "TLB", "CF", "CTL")

#------------------------------------------------------------------------------
#  "EnergyTransaction" class:
#------------------------------------------------------------------------------

class EnergyTransaction(Document):
    """ Specifies the schedule for energy transfers between interchange areas that are necessary to satisfy the associated interchange transaction.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # { Approve | Deny | Study }
    state = Instance("CIM.EnumeratedType",
        desc="{ Approve | Deny | Study }",
        transient=True,
        editor=InstanceEditor(name="_enumeratedtypes"))

    def _get_enumeratedtypes(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.EnumeratedType" ]
        else:
            return []

    _enumeratedtypes = Property(fget=_get_enumeratedtypes)

    # An EnergyTransaction may be curtailed by any of the participating entities.
    CurtailmentProfiles = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.CurtailmentProfile"),
        desc="An EnergyTransaction may be curtailed by any of the participating entities.")

    # Energy is transferred between interchange areas
    Export_SubControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.SubControlArea",
        desc="Energy is transferred between interchange areas",
        transient=True,
        opposite="Export_EnergyTransactions",
        editor=InstanceEditor(name="_subcontrolareas"))

    def _get_subcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.SubControlArea" ]
        else:
            return []

    _subcontrolareas = Property(fget=_get_subcontrolareas)

    EnergyTransId = List(Instance("CIM.IEC61968.Informative.MarketOperations.TransactionBid"))

    # Energy is transferred between interchange areas
    Import_SubControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.SubControlArea",
        desc="Energy is transferred between interchange areas",
        transient=True,
        opposite="Import_EnergyTransactions",
        editor=InstanceEditor(name="_subcontrolareas"))

    def _get_subcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.SubControlArea" ]
        else:
            return []

    _subcontrolareas = Property(fget=_get_subcontrolareas)

    EnergyPriceCurves = List(Instance("CIM.IEC61968.Informative.MarketOperations.EnergyPriceCurve"))

    # An EnergyTransaction may have a LossProfile.
    LossProfiles = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.LossProfile"),
        desc="An EnergyTransaction may have a LossProfile.")

    # An EnergyTransaction must have at least one EnergyProfile.
    EnergyProfiles = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyProfile"),
        desc="An EnergyTransaction must have at least one EnergyProfile.")

    # The 'Source' for an EnergyTransaction is an EnergyProduct which is injected into a ControlArea.
    EnergyProduct = Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyProduct",
        desc="The 'Source' for an EnergyTransaction is an EnergyProduct which is injected into a ControlArea.",
        transient=True,
        opposite="EnergyTransactions",
        editor=InstanceEditor(name="_energyproducts"))

    def _get_energyproducts(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.EnergyProduct" ]
        else:
            return []

    _energyproducts = Property(fget=_get_energyproducts)

    # Receipt point active power
    receiptPointP = ActivePower(desc="Receipt point active power")

    # Transaction minimum active power if dispatchable
    energyMin = ActivePower(desc="Transaction minimum active power if dispatchable")

    # Firm interchange flag indicates whether or not this energy transaction can be changed without potential financial consequences.
    firmInterchangeFlag = Bool(desc="Firm interchange flag indicates whether or not this energy transaction can be changed without potential financial consequences.")

    # Maximum congestion charges in monetary units
    congestChargeMax = Money(desc="Maximum congestion charges in monetary units")

    reason = Str

    # Delivery point active power
    deliveryPointP = ActivePower(desc="Delivery point active power")

    #--------------------------------------------------------------------------
    #  Begin "EnergyTransaction" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "receiptPointP", "energyMin", "firmInterchangeFlag", "congestChargeMax", "reason", "deliveryPointP",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "state", "CurtailmentProfiles", "Export_SubControlArea", "EnergyTransId", "Import_SubControlArea", "EnergyPriceCurves", "LossProfiles", "EnergyProfiles", "EnergyProduct",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction",
        title="EnergyTransaction",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EnergyTransaction" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "AvailableTransmissionCapacity" class:
#------------------------------------------------------------------------------

class AvailableTransmissionCapacity(Curve):
    """ Amount of possible flow by direction.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A transmission schedule posts the available transmission capacity for a transmission line.
    ScheduleFor = List(Instance("CIM.IEC61968.Informative.Reservation.TransmissionService"),
        desc="A transmission schedule posts the available transmission capacity for a transmission line.")

    #--------------------------------------------------------------------------
    #  Begin "AvailableTransmissionCapacity" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "y3Multiplier", "y2Multiplier", "xMultiplier", "y2Unit", "curveStyle", "y1Unit", "y1Multiplier", "y3Unit", "xUnit",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "CurveDatas", "ScheduleFor",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.AvailableTransmissionCapacity",
        title="AvailableTransmissionCapacity",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "AvailableTransmissionCapacity" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ProfileData" class:
#------------------------------------------------------------------------------

class ProfileData(Element):
    """ Data for profile.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A profile has profile data associated with it.
    Profile = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.Profile"),
        desc="A profile has profile data associated with it.")

    # Stop date/time for this profile.
    stopDateTime = Date(desc="Stop date/time for this profile.")

    # Energy level for the profile
    energyLevel = RealEnergy(desc="Energy level for the profile")

    # Start date/time for this profile.
    startDateTime = Date(desc="Start date/time for this profile.")

    # Active power capacity level for the profile.
    capacityLevel = ActivePower(desc="Active power capacity level for the profile.")

    # Sequence to provide item numbering for the profile. { greater than or equal to 1 }
    sequenceNumber = Int(desc="Sequence to provide item numbering for the profile. { greater than or equal to 1 }")

    #--------------------------------------------------------------------------
    #  Begin "ProfileData" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "stopDateTime", "energyLevel", "startDateTime", "capacityLevel", "sequenceNumber",
                label="Attributes"),
            VGroup("Parent", "Profile",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.ProfileData",
        title="ProfileData",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ProfileData" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Profile" class:
#------------------------------------------------------------------------------

class Profile(IdentifiedObject):
    """ A profile is a simpler curve type.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A profile has profile data associated with it.
    ProfileDatas = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.ProfileData"),
        desc="A profile has profile data associated with it.")

    #--------------------------------------------------------------------------
    #  Begin "Profile" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ProfileDatas",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.Profile",
        title="Profile",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Profile" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SubControlArea" class:
#------------------------------------------------------------------------------

class SubControlArea(ControlArea):
    """ SubControlArea replacement classed moved into EnergySchedulingPackage.  An area defined for the purpose of tracking interchange with surrounding areas via tie points; may or may not serve as a control area.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A control area may own 0 to n flowgates A flowgate must be owned by exactly 1 control area
    Flowgate = List(Instance("CIM.IEC61968.Informative.MarketOperations.Flowgate"),
        desc="A control area may own 0 to n flowgates A flowgate must be owned by exactly 1 control area")

    # Energy is transferred between interchange areas
    Export_EnergyTransactions = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction"),
        desc="Energy is transferred between interchange areas")

    # Energy is transferred between interchange areas
    Import_EnergyTransactions = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction"),
        desc="Energy is transferred between interchange areas")

    # The interchange area  may operate as a control area
    HostControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.HostControlArea",
        desc="The interchange area  may operate as a control area",
        transient=True,
        opposite="SubControlAreas",
        editor=InstanceEditor(name="_hostcontrolareas"))

    def _get_hostcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.HostControlArea" ]
        else:
            return []

    _hostcontrolareas = Property(fget=_get_hostcontrolareas)

    # A transmission path's service point is part of an interchange area
    PartOf = List(Instance("CIM.IEC61968.Informative.Reservation.ServicePoint"),
        desc="A transmission path's service point is part of an interchange area")

    # The SubControlArea is on the A side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.
    SideA_TieLines = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.TieLine"),
        desc="The SubControlArea is on the A side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.")

    # A GeneratingUnit injects energy into a SubControlArea.
    GeneratingUnits = List(Instance("CIM.IEC61970.Generation.Production.GeneratingUnit"),
        desc="A GeneratingUnit injects energy into a SubControlArea.")

    # The SubControlArea is on the B side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.
    SideB_TieLines = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.TieLine"),
        desc="The SubControlArea is on the B side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.")

    #--------------------------------------------------------------------------
    #  Begin "SubControlArea" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "type", "pTolerance", "netInterchange",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ChangeItems", "AssetRoles", "GeoLocation", "SafetyDocuments", "OutageSchedule", "Measurements", "ErpOrganisationRoles", "PSRType", "PsrLists", "PSREvent", "OperatingShare", "ScheduleSteps", "DocumentRoles", "ReportingGroup", "CircuitSections", "NetworkDataSets", "EnergyArea", "ControlAreaGeneratingUnit", "TieFlow", "Flowgate", "Export_EnergyTransactions", "Import_EnergyTransactions", "HostControlArea", "PartOf", "SideA_TieLines", "GeneratingUnits", "SideB_TieLines",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.SubControlArea",
        title="SubControlArea",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SubControlArea" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TransmissionRightOfWay" class:
#------------------------------------------------------------------------------

class TransmissionRightOfWay(PowerSystemResource):
    """ A collection of transmission lines that are close proximity to each other.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A transmission right-of-way is a member of a transmission corridor
    TransmissionCorridor = Instance("CIM.IEC61968.Informative.EnergyScheduling.TransmissionCorridor",
        desc="A transmission right-of-way is a member of a transmission corridor",
        transient=True,
        opposite="TransmissionRightOfWays",
        editor=InstanceEditor(name="_transmissioncorridors"))

    def _get_transmissioncorridors(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.TransmissionCorridor" ]
        else:
            return []

    _transmissioncorridors = Property(fget=_get_transmissioncorridors)

    # A transmission line can be part of a transmission corridor
    Lines = List(Instance("CIM.IEC61970.Wires.Line"),
        desc="A transmission line can be part of a transmission corridor")

    #--------------------------------------------------------------------------
    #  Begin "TransmissionRightOfWay" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ChangeItems", "AssetRoles", "GeoLocation", "SafetyDocuments", "OutageSchedule", "Measurements", "ErpOrganisationRoles", "PSRType", "PsrLists", "PSREvent", "OperatingShare", "ScheduleSteps", "DocumentRoles", "ReportingGroup", "CircuitSections", "NetworkDataSets", "TransmissionCorridor", "Lines",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.TransmissionRightOfWay",
        title="TransmissionRightOfWay",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TransmissionRightOfWay" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EnergySchedulingVersion" class:
#------------------------------------------------------------------------------

class EnergySchedulingVersion(Element):
    date = Date

    # v 4 moved SubControlArea
    version = Str(desc="v 4 moved SubControlArea")

    #--------------------------------------------------------------------------
    #  Begin "EnergySchedulingVersion" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "date", "version",
                label="Attributes"),
            VGroup("Parent",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.EnergySchedulingVersion",
        title="EnergySchedulingVersion",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EnergySchedulingVersion" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "HostControlArea" class:
#------------------------------------------------------------------------------

class HostControlArea(IdentifiedObject):
    """ A HostControlArea has a set of tie points and a set of generator controls (i.e., AGC). It also has a total load, including transmission and distribution losses.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A control area can have one or more net inadvertent interchange accounts
    InadvertentAccounts = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.InadvertentAccount"),
        desc="A control area can have one or more net inadvertent interchange accounts")

    # A control area has one or more area reserve specifications
    AreaReserveSpec = Instance("CIM.IEC61968.Informative.EnergyScheduling.AreaReserveSpec",
        desc="A control area has one or more area reserve specifications",
        transient=True,
        opposite="HostControlAreas",
        editor=InstanceEditor(name="_areareservespecs"))

    def _get_areareservespecs(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.AreaReserveSpec" ]
        else:
            return []

    _areareservespecs = Property(fget=_get_areareservespecs)

    # The control area's frequency bias factor, in active power per frequency, for automatic generation control (AGC)
    frequencyBiasFactor = Instance("CIM.FreqBiasFactor",
        desc="The control area's frequency bias factor, in active power per frequency, for automatic generation control (AGC)",
        transient=True,
        editor=InstanceEditor(name="_freqbiasfactors"))

    def _get_freqbiasfactors(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.FreqBiasFactor" ]
        else:
            return []

    _freqbiasfactors = Property(fget=_get_freqbiasfactors)

    # A HostControlArea can have zero or more tie lines.
    SideA_TieLines = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.TieLine"),
        desc="A HostControlArea can have zero or more tie lines.")

    # The interchange area  may operate as a control area
    SubControlAreas = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.SubControlArea"),
        desc="The interchange area  may operate as a control area")

    # A HostControlArea can have zero or more tie lines.
    SideB_TieLines = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.TieLine"),
        desc="A HostControlArea can have zero or more tie lines.")

    # A control area can receive dynamic schedules from other control areas
    Receive_DynamicSchedules = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.DynamicSchedule"),
        desc="A control area can receive dynamic schedules from other control areas")

    # A control area can send dynamic schedules to other control areas
    Send_DynamicSchedules = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.DynamicSchedule"),
        desc="A control area can send dynamic schedules to other control areas")

    # A ControlAreaCompany controls a ControlArea.
    Controls = Instance("CIM.IEC61968.Informative.Financial.ControlAreaOperator",
        desc="A ControlAreaCompany controls a ControlArea.",
        transient=True,
        opposite="ControlledBy",
        editor=InstanceEditor(name="_controlareaoperators"))

    def _get_controlareaoperators(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.Financial.ControlAreaOperator" ]
        else:
            return []

    _controlareaoperators = Property(fget=_get_controlareaoperators)

    # The area's present control mode: (CF = constant frequency) or (CTL = constant tie-line) or (TLB = tie-line bias) or (OFF = off control)
    areaControlMode = AreaControlMode(desc="The area's present control mode: (CF = constant frequency) or (CTL = constant tie-line) or (TLB = tie-line bias) or (OFF = off control)")

    # The present power system frequency set point for automatic generation control
    freqSetPoint = Frequency(desc="The present power system frequency set point for automatic generation control")

    #--------------------------------------------------------------------------
    #  Begin "HostControlArea" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "areaControlMode", "freqSetPoint",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "InadvertentAccounts", "AreaReserveSpec", "frequencyBiasFactor", "SideA_TieLines", "SubControlAreas", "SideB_TieLines", "Receive_DynamicSchedules", "Send_DynamicSchedules", "Controls",
                label="References", columns=1),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.HostControlArea",
        title="HostControlArea",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "HostControlArea" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "InadvertentAccount" class:
#------------------------------------------------------------------------------

class InadvertentAccount(Curve):
    """ An account for tracking inadvertent interchange versus time for each control area. A control area may have more than one inadvertent account in order to track inadvertent over one or more specific tie points in addition to the usual overall net inadvertent. Separate accounts would also be used to track designated time periods, such as on-peak and off-peak.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A control area can have one or more net inadvertent interchange accounts
    HostControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.HostControlArea",
        desc="A control area can have one or more net inadvertent interchange accounts",
        transient=True,
        opposite="InadvertentAccounts",
        editor=InstanceEditor(name="_hostcontrolareas"))

    def _get_hostcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.HostControlArea" ]
        else:
            return []

    _hostcontrolareas = Property(fget=_get_hostcontrolareas)

    #--------------------------------------------------------------------------
    #  Begin "InadvertentAccount" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "y3Multiplier", "y2Multiplier", "xMultiplier", "y2Unit", "curveStyle", "y1Unit", "y1Multiplier", "y3Unit", "xUnit",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "CurveDatas", "HostControlArea",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.InadvertentAccount",
        title="InadvertentAccount",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "InadvertentAccount" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TransmissionCorridor" class:
#------------------------------------------------------------------------------

class TransmissionCorridor(PowerSystemResource):
    """ A corridor containing one or more rights of way
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A transmission right-of-way is a member of a transmission corridor
    TransmissionRightOfWays = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.TransmissionRightOfWay"),
        desc="A transmission right-of-way is a member of a transmission corridor")

    # A TransmissionPath is contained in a TransmissionCorridor.
    ContainedIn = List(Instance("CIM.IEC61968.Informative.Reservation.TransmissionPath"),
        desc="A TransmissionPath is contained in a TransmissionCorridor.")

    #--------------------------------------------------------------------------
    #  Begin "TransmissionCorridor" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ChangeItems", "AssetRoles", "GeoLocation", "SafetyDocuments", "OutageSchedule", "Measurements", "ErpOrganisationRoles", "PSRType", "PsrLists", "PSREvent", "OperatingShare", "ScheduleSteps", "DocumentRoles", "ReportingGroup", "CircuitSections", "NetworkDataSets", "TransmissionRightOfWays", "ContainedIn",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.TransmissionCorridor",
        title="TransmissionCorridor",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TransmissionCorridor" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "AreaReserveSpec" class:
#------------------------------------------------------------------------------

class AreaReserveSpec(Element):
    """ The control area's reserve specification
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A control area has one or more area reserve specifications
    HostControlAreas = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.HostControlArea"),
        desc="A control area has one or more area reserve specifications")

    # A Reserve type of energy transaction can count towards an area reserve specification.
    ReserveEnergyTransaction = Instance("CIM.IEC61968.Informative.EnergyScheduling.Reserve",
        desc="A Reserve type of energy transaction can count towards an area reserve specification.",
        transient=True,
        opposite="AreaReserveSpec",
        editor=InstanceEditor(name="_reserves"))

    def _get_reserves(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.Reserve" ]
        else:
            return []

    _reserves = Property(fget=_get_reserves)

    # The Identification or name of the control area's reserve specification. A particular specification could correspond to pre-defined power system conditions, e.g., emergency situations.
    areaReserveSpecName = Str(desc="The Identification or name of the control area's reserve specification. A particular specification could correspond to pre-defined power system conditions, e.g., emergency situations.")

    # Raise active power regulating margin requirement, the amount of generation that can be picked up by control in 10 minutes
    raiseRegMarginReqt = ActivePower(desc="Raise active power regulating margin requirement, the amount of generation that can be picked up by control in 10 minutes")

    # Lower active power regulating margin requirement, the amount of generation that can be dropped by control in 10 minutes
    lowerRegMarginReqt = ActivePower(desc="Lower active power regulating margin requirement, the amount of generation that can be dropped by control in 10 minutes")

    # Primary active power reserve requirement, where primary reserve is generating capability that is fully available within 10 minutes. Primary reserve is composed of spinning reserve and quick-start reserve.
    primaryReserveReqt = ActivePower(desc="Primary active power reserve requirement, where primary reserve is generating capability that is fully available within 10 minutes. Primary reserve is composed of spinning reserve and quick-start reserve.")

    # Operating active power reserve requirement, where operating reserve is the generating capability that is fully available within 30 minutes. Operating reserve is composed of primary reserve (t less than 10 min) and secondary reserve (10 less than t less than 30 min).
    opReserveReqt = ActivePower(desc="Operating active power reserve requirement, where operating reserve is the generating capability that is fully available within 30 minutes. Operating reserve is composed of primary reserve (t less than 10 min) and secondary reserve (10 less than t less than 30 min).")

    # Spinning active power reserve requirement, spinning reserve is generating capability that is presently synchronized to the network and is fully available within 10 minutes
    spinningReserveReqt = ActivePower(desc="Spinning active power reserve requirement, spinning reserve is generating capability that is presently synchronized to the network and is fully available within 10 minutes")

    #--------------------------------------------------------------------------
    #  Begin "AreaReserveSpec" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "areaReserveSpecName", "raiseRegMarginReqt", "lowerRegMarginReqt", "primaryReserveReqt", "opReserveReqt", "spinningReserveReqt",
                label="Attributes"),
            VGroup("Parent", "HostControlAreas", "ReserveEnergyTransaction",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.AreaReserveSpec",
        title="AreaReserveSpec",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "AreaReserveSpec" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EnergyProduct" class:
#------------------------------------------------------------------------------

class EnergyProduct(Agreement):
    """ An EnergyProduct is offered commercially as a ContractOrTariff.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # An EnergyProduct injects energy into a service point.
    ServicePoint = List(Instance("CIM.IEC61968.Informative.Reservation.ServicePoint"),
        desc="An EnergyProduct injects energy into a service point.")

    GenerationProvider = Instance("CIM.IEC61968.Informative.Financial.GenerationProvider",
        transient=True,
        opposite="EnergyProducts",
        editor=InstanceEditor(name="_generationproviders"))

    def _get_generationproviders(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.Financial.GenerationProvider" ]
        else:
            return []

    _generationproviders = Property(fget=_get_generationproviders)

    # The 'Source' for an EnergyTransaction is an EnergyProduct which is injected into a ControlArea.
    EnergyTransactions = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction"),
        desc="The 'Source' for an EnergyTransaction is an EnergyProduct which is injected into a ControlArea.")

    # A Marketer may resell an EnergyProduct.
    ResoldBy_Marketers = List(Instance("CIM.IEC61968.Informative.Financial.Marketer"),
        desc="A Marketer may resell an EnergyProduct.")

    # A Marketer holds title to an EnergyProduct.
    TitleHeldBy_Marketer = Instance("CIM.IEC61968.Informative.Financial.Marketer",
        desc="A Marketer holds title to an EnergyProduct.",
        transient=True,
        opposite="HoldsTitleTo_EnergyProducts",
        editor=InstanceEditor(name="_marketers"))

    def _get_marketers(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.Financial.Marketer" ]
        else:
            return []

    _marketers = Property(fget=_get_marketers)

    #--------------------------------------------------------------------------
    #  Begin "EnergyProduct" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "signDate",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "validityInterval", "ServicePoint", "GenerationProvider", "EnergyTransactions", "ResoldBy_Marketers", "TitleHeldBy_Marketer",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.EnergyProduct",
        title="EnergyProduct",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EnergyProduct" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TieLine" class:
#------------------------------------------------------------------------------

class TieLine(Element):
    # A HostControlArea can have zero or more tie lines.
    SideA_HostControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.HostControlArea",
        desc="A HostControlArea can have zero or more tie lines.",
        transient=True,
        opposite="SideA_TieLines",
        editor=InstanceEditor(name="_hostcontrolareas"))

    def _get_hostcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.HostControlArea" ]
        else:
            return []

    _hostcontrolareas = Property(fget=_get_hostcontrolareas)

    # A HostControlArea can have zero or more tie lines.
    SideB_HostControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.HostControlArea",
        desc="A HostControlArea can have zero or more tie lines.",
        transient=True,
        opposite="SideB_TieLines",
        editor=InstanceEditor(name="_hostcontrolareas"))

    def _get_hostcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.HostControlArea" ]
        else:
            return []

    _hostcontrolareas = Property(fget=_get_hostcontrolareas)

    # The SubControlArea is on the A side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.
    SideA_SubControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.SubControlArea",
        desc="The SubControlArea is on the A side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.",
        transient=True,
        opposite="SideA_TieLines",
        editor=InstanceEditor(name="_subcontrolareas"))

    def _get_subcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.SubControlArea" ]
        else:
            return []

    _subcontrolareas = Property(fget=_get_subcontrolareas)

    # A CustomerConsumer may ring its perimeter with metering, which can create a unique SubControlArea at the collection of metering points, called a TieLine.
    CustomerConsumer = Instance("CIM.IEC61968.Informative.Financial.CustomerConsumer",
        desc="A CustomerConsumer may ring its perimeter with metering, which can create a unique SubControlArea at the collection of metering points, called a TieLine.",
        transient=True,
        opposite="TieLines",
        editor=InstanceEditor(name="_customerconsumers"))

    def _get_customerconsumers(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.Financial.CustomerConsumer" ]
        else:
            return []

    _customerconsumers = Property(fget=_get_customerconsumers)

    # The SubControlArea is on the B side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.
    SideB_SubControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.SubControlArea",
        desc="The SubControlArea is on the B side of a collection of metered points which define the SubControlArea's boundary for a ControlAreaOperator or CustomerConsumer.",
        transient=True,
        opposite="SideB_TieLines",
        editor=InstanceEditor(name="_subcontrolareas"))

    def _get_subcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.SubControlArea" ]
        else:
            return []

    _subcontrolareas = Property(fget=_get_subcontrolareas)

    # A dynamic energy transaction can act as a pseudo tie line.
    DynamicEnergyTransaction = Instance("CIM.IEC61968.Informative.EnergyScheduling.Dynamic",
        desc="A dynamic energy transaction can act as a pseudo tie line.",
        transient=True,
        opposite="TieLines",
        editor=InstanceEditor(name="_dynamics"))

    def _get_dynamics(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.Dynamic" ]
        else:
            return []

    _dynamics = Property(fget=_get_dynamics)

    # A ControlAreaOperator has a collection of tie points that ring the ControlArea, called a TieLine.
    ControlAreaOperators = List(Instance("CIM.IEC61968.Informative.Financial.ControlAreaOperator"),
        desc="A ControlAreaOperator has a collection of tie points that ring the ControlArea, called a TieLine.")

    #--------------------------------------------------------------------------
    #  Begin "TieLine" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID",
                label="Attributes"),
            VGroup("Parent", "SideA_HostControlArea", "SideB_HostControlArea", "SideA_SubControlArea", "CustomerConsumer", "SideB_SubControlArea", "DynamicEnergyTransaction", "ControlAreaOperators",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.TieLine",
        title="TieLine",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TieLine" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DynamicSchedule" class:
#------------------------------------------------------------------------------

class DynamicSchedule(RegularIntervalSchedule):
    """ A continuously variable component of a control area's active power net interchange schedule. Dynamic schedules are sent and received by control areas.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A measurement is a data source for dynamic interchange schedules
    Measurement = Instance("CIM.IEC61970.Meas.Measurement",
        desc="A measurement is a data source for dynamic interchange schedules",
        transient=True,
        opposite="DynamicSchedules",
        editor=InstanceEditor(name="_measurements"))

    def _get_measurements(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Meas.Measurement" ]
        else:
            return []

    _measurements = Property(fget=_get_measurements)

    # A control area can receive dynamic schedules from other control areas
    Receive_HostControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.HostControlArea",
        desc="A control area can receive dynamic schedules from other control areas",
        transient=True,
        opposite="Receive_DynamicSchedules",
        editor=InstanceEditor(name="_hostcontrolareas"))

    def _get_hostcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.HostControlArea" ]
        else:
            return []

    _hostcontrolareas = Property(fget=_get_hostcontrolareas)

    # A control area can send dynamic schedules to other control areas
    Send_HostControlArea = Instance("CIM.IEC61968.Informative.EnergyScheduling.HostControlArea",
        desc="A control area can send dynamic schedules to other control areas",
        transient=True,
        opposite="Send_DynamicSchedules",
        editor=InstanceEditor(name="_hostcontrolareas"))

    def _get_hostcontrolareas(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.HostControlArea" ]
        else:
            return []

    _hostcontrolareas = Property(fget=_get_hostcontrolareas)

    # The 'active' or 'inactive' status of the dynamic schedule
    dynSchedStatus = Str(desc="The 'active' or 'inactive' status of the dynamic schedule")

    # Dynamic schedule sign reversal required (yes/no)
    dynSchedSignRev = Bool(desc="Dynamic schedule sign reversal required (yes/no)")

    #--------------------------------------------------------------------------
    #  Begin "DynamicSchedule" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "value2Multiplier", "value1Unit", "value2Unit", "value1Multiplier", "startTime", "timeStep", "endTime", "dynSchedStatus", "dynSchedSignRev",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "TimePoints", "Measurement", "Receive_HostControlArea", "Send_HostControlArea",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.DynamicSchedule",
        title="DynamicSchedule",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DynamicSchedule" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Reserve" class:
#------------------------------------------------------------------------------

class Reserve(EnergyTransaction):
    # A Reserve type of energy transaction can count towards an area reserve specification.
    AreaReserveSpec = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.AreaReserveSpec"),
        desc="A Reserve type of energy transaction can count towards an area reserve specification.")

    #--------------------------------------------------------------------------
    #  Begin "Reserve" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "receiptPointP", "energyMin", "firmInterchangeFlag", "congestChargeMax", "reason", "deliveryPointP",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "state", "CurtailmentProfiles", "Export_SubControlArea", "EnergyTransId", "Import_SubControlArea", "EnergyPriceCurves", "LossProfiles", "EnergyProfiles", "EnergyProduct", "AreaReserveSpec",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.Reserve",
        title="Reserve",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Reserve" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Block" class:
#------------------------------------------------------------------------------

class Block(EnergyTransaction):
    """ A block is a simple transaction type, with no additional relationships.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "Block" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "receiptPointP", "energyMin", "firmInterchangeFlag", "congestChargeMax", "reason", "deliveryPointP",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "state", "CurtailmentProfiles", "Export_SubControlArea", "EnergyTransId", "Import_SubControlArea", "EnergyPriceCurves", "LossProfiles", "EnergyProfiles", "EnergyProduct",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.Block",
        title="Block",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Block" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "LossProfile" class:
#------------------------------------------------------------------------------

class LossProfile(Profile):
    """ LossProfile is associated with an EnerrgyTransaction and must be completely contained within the time frame of the EnergyProfile associated with this EnergyTransaction.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # An EnergyTransaction may have a LossProfile.
    EnergyTransaction = Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction",
        desc="An EnergyTransaction may have a LossProfile.",
        transient=True,
        opposite="LossProfiles",
        editor=InstanceEditor(name="_energytransactions"))

    def _get_energytransactions(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction" ]
        else:
            return []

    _energytransactions = Property(fget=_get_energytransactions)

    # Part of the LossProfile for an EnergyTransaction may be a loss for a TransmissionProvider.
    HasLoss_ = Instance("CIM.IEC61968.Informative.Financial.TransmissionProvider",
        desc="Part of the LossProfile for an EnergyTransaction may be a loss for a TransmissionProvider.",
        transient=True,
        opposite="For",
        editor=InstanceEditor(name="_transmissionproviders"))

    def _get_transmissionproviders(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.Financial.TransmissionProvider" ]
        else:
            return []

    _transmissionproviders = Property(fget=_get_transmissionproviders)

    #--------------------------------------------------------------------------
    #  Begin "LossProfile" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ProfileDatas", "EnergyTransaction", "HasLoss_",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.LossProfile",
        title="LossProfile",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LossProfile" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CurtailmentProfile" class:
#------------------------------------------------------------------------------

class CurtailmentProfile(Profile):
    """ Curtailing entity must be providing at least one service to the EnergyTransaction. The CurtailmentProfile must be completely contained within the EnergyProfile timeframe for this EnergyTransaction.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # An EnergyTransaction may be curtailed by any of the participating entities.
    EnergyTransaction = Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction",
        desc="An EnergyTransaction may be curtailed by any of the participating entities.",
        transient=True,
        opposite="CurtailmentProfiles",
        editor=InstanceEditor(name="_energytransactions"))

    def _get_energytransactions(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction" ]
        else:
            return []

    _energytransactions = Property(fget=_get_energytransactions)

    #--------------------------------------------------------------------------
    #  Begin "CurtailmentProfile" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ProfileDatas", "EnergyTransaction",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.CurtailmentProfile",
        title="CurtailmentProfile",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CurtailmentProfile" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Dynamic" class:
#------------------------------------------------------------------------------

class Dynamic(EnergyTransaction):
    """ A dynamic energy transaction has more complex relationships than a simple block type. It behaves like a pseudo tie line.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A dynamic energy transaction can act as a pseudo tie line.
    TieLines = List(Instance("CIM.IEC61968.Informative.EnergyScheduling.TieLine"),
        desc="A dynamic energy transaction can act as a pseudo tie line.")

    #--------------------------------------------------------------------------
    #  Begin "Dynamic" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "receiptPointP", "energyMin", "firmInterchangeFlag", "congestChargeMax", "reason", "deliveryPointP",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "state", "CurtailmentProfiles", "Export_SubControlArea", "EnergyTransId", "Import_SubControlArea", "EnergyPriceCurves", "LossProfiles", "EnergyProfiles", "EnergyProduct", "TieLines",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.Dynamic",
        title="Dynamic",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Dynamic" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EnergyProfile" class:
#------------------------------------------------------------------------------

class EnergyProfile(Profile):
    """ Specifies the start time, stop time, level for an EnergyTransaction.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    TransactionBid = Instance("CIM.IEC61968.Informative.MarketOperations.TransactionBid",
        transient=True,
        opposite="EnergyProfiles",
        editor=InstanceEditor(name="_transactionbids"))

    def _get_transactionbids(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.MarketOperations.TransactionBid" ]
        else:
            return []

    _transactionbids = Property(fget=_get_transactionbids)

    # An EnergyTransaction must have at least one EnergyProfile.
    EnergyTransaction = Instance("CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction",
        desc="An EnergyTransaction must have at least one EnergyProfile.",
        transient=True,
        opposite="EnergyProfiles",
        editor=InstanceEditor(name="_energytransactions"))

    def _get_energytransactions(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.EnergyScheduling.EnergyTransaction" ]
        else:
            return []

    _energytransactions = Property(fget=_get_energytransactions)

    #--------------------------------------------------------------------------
    #  Begin "EnergyProfile" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ProfileDatas", "TransactionBid", "EnergyTransaction",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.EnergyScheduling.EnergyProfile",
        title="EnergyProfile",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EnergyProfile" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" The OperationalLimits package models a specification of limits associated with equipment and other operational entities.The OperationalLimits package models a specification of limits associated with equipment and other operational entities.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from UCTE.Core import IdentifiedObject
from UCTE.Domain import CurrentFlow
from UCTE.Domain import Voltage
from UCTE.Domain import Seconds



from enthought.traits.api import Instance, List, Property, Enum
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


OperationalLimitDirectionKind = Enum("high", "absoluteValue", "low")

#------------------------------------------------------------------------------
#  "OperationalLimit" class:
#------------------------------------------------------------------------------

class OperationalLimit(IdentifiedObject):
    """ A value associated with a specific kind of limit.A value associated with a specific kind of limit.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The limit set to which the limit values belong.The limit set to which the limit values belong.
    OperationalLimitSet = Instance("UCTE.OperationalLimits.OperationalLimitSet", allow_none=False,
        desc="The limit set to which the limit values belong.The limit set to which the limit values belong.",
        transient=True,
        opposite="OperationalLimitValue",
        editor=InstanceEditor(name="_operationallimitsets"))

    def _get_operationallimitsets(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.OperationalLimits.OperationalLimitSet" ]
        else:
            return []

    _operationallimitsets = Property(fget=_get_operationallimitsets)

    # The limit type associated with this limit.The limit type associated with this limit.
    OperationalLimitType = Instance("UCTE.OperationalLimits.OperationalLimitType", allow_none=False,
        desc="The limit type associated with this limit.The limit type associated with this limit.",
        transient=True,
        opposite="OperationalLimit",
        editor=InstanceEditor(name="_operationallimittypes"))

    def _get_operationallimittypes(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.OperationalLimits.OperationalLimitType" ]
        else:
            return []

    _operationallimittypes = Property(fget=_get_operationallimittypes)

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "name", "aliasName",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet", "OperationalLimitType",
                label="References"),
            dock="tab"),
        id="UCTE.OperationalLimits.OperationalLimit",
        title="OperationalLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OperationalLimitSet" class:
#------------------------------------------------------------------------------

class OperationalLimitSet(IdentifiedObject):
    """ A set of limits associated with equipmnet.  Sets of limits might apply to a specific temperature, or season for example. A set of limits may contain may different severiteis of limit levels that would apply to the same equipment.   The set may contain limits of different types such as apparent power and current limits or high and low voltage limits  that are logically applied together as a set.A set of limits associated with equipmnet.  Sets of limits might apply to a specific temperature, or season for example. A set of limits may contain may different severiteis of limit levels that would apply to the same equipment.   The set may contain limits of different types such as apparent power and current limits or high and low voltage limits  that are logically applied together as a set.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The terminal specifically associated to this operational limit set.  If no terminal is associated, all terminals of the equipment are implied.For UCTE profile, the terminal associated with the limit is always required, and thus there is no need to exchange the associated Equipment which can always be derived from the terminal.The terminal specifically associated to this operational limit set.  If no terminal is associated, all terminals of the equipment are implied.For UCTE profile, the terminal associated with the limit is always required, and thus there is no need to exchange the associated Equipment which can always be derived from the terminal.
    Terminal = Instance("UCTE.Core.Terminal", allow_none=False,
        desc="The terminal specifically associated to this operational limit set.  If no terminal is associated, all terminals of the equipment are implied.For UCTE profile, the terminal associated with the limit is always required, and thus there is no need to exchange the associated Equipment which can always be derived from the terminal.The terminal specifically associated to this operational limit set.  If no terminal is associated, all terminals of the equipment are implied.For UCTE profile, the terminal associated with the limit is always required, and thus there is no need to exchange the associated Equipment which can always be derived from the terminal.",
        transient=True,
        opposite="OperationalLimitSet",
        editor=InstanceEditor(name="_terminals"))

    def _get_terminals(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Core.Terminal" ]
        else:
            return []

    _terminals = Property(fget=_get_terminals)

    # Values of equipment limits.Values of equipment limits.
    OperationalLimitValue = List(Instance("UCTE.OperationalLimits.OperationalLimit"),
        desc="Values of equipment limits.Values of equipment limits.")

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimitSet" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "name", "aliasName",
                label="Attributes"),
            VGroup("Model", "Terminal", "OperationalLimitValue",
                label="References"),
            dock="tab"),
        id="UCTE.OperationalLimits.OperationalLimitSet",
        title="OperationalLimitSet",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimitSet" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OperationalLimitType" class:
#------------------------------------------------------------------------------

class OperationalLimitType(IdentifiedObject):
    """ A type of limit.  The meaning of a specific limit is described in this class.A type of limit.  The meaning of a specific limit is described in this class.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The operational limits associated with this type of limit.The operational limits associated with this type of limit.
    OperationalLimit = List(Instance("UCTE.OperationalLimits.OperationalLimit"),
        desc="The operational limits associated with this type of limit.The operational limits associated with this type of limit.")

    # The direction of the limit.The direction of the limit.
    direction = OperationalLimitDirectionKind(desc="The direction of the limit.The direction of the limit.")

    # The nominal acceptable duration of the limit.  Limits are commonly expressed in terms of the a time limit for which the limit is normally acceptable.   The actual acceptable duration of a specific limit may depend on other local factors such as temperature or wind speed.The nominal acceptable duration of the limit.  Limits are commonly expressed in terms of the a time limit for which the limit is normally acceptable.   The actual acceptable duration of a specific limit may depend on other local factors such as temperature or wind speed.
    acceptableDuration = Seconds(desc="The nominal acceptable duration of the limit.  Limits are commonly expressed in terms of the a time limit for which the limit is normally acceptable.   The actual acceptable duration of a specific limit may depend on other local factors such as temperature or wind speed.The nominal acceptable duration of the limit.  Limits are commonly expressed in terms of the a time limit for which the limit is normally acceptable.   The actual acceptable duration of a specific limit may depend on other local factors such as temperature or wind speed.")

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimitType" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "name", "aliasName", "direction", "acceptableDuration",
                label="Attributes"),
            VGroup("Model", "OperationalLimit",
                label="References"),
            dock="tab"),
        id="UCTE.OperationalLimits.OperationalLimitType",
        title="OperationalLimitType",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimitType" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CurrentLimit" class:
#------------------------------------------------------------------------------

class CurrentLimit(OperationalLimit):
    """ Operational limit on current.Operational limit on current.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Limit on current flow.Limit on current flow.
    value = CurrentFlow(desc="Limit on current flow.Limit on current flow.")

    #--------------------------------------------------------------------------
    #  Begin "CurrentLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "name", "aliasName", "value",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet", "OperationalLimitType",
                label="References"),
            dock="tab"),
        id="UCTE.OperationalLimits.CurrentLimit",
        title="CurrentLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CurrentLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "VoltageLimit" class:
#------------------------------------------------------------------------------

class VoltageLimit(OperationalLimit):
    """ Operational limit applied to voltage.Operational limit applied to voltage.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Limit on voltage. High or low limit depends on the OperatoinalLimit.limitKindLimit on voltage. High or low limit depends on the OperatoinalLimit.limitKind
    value = Voltage(desc="Limit on voltage. High or low limit depends on the OperatoinalLimit.limitKindLimit on voltage. High or low limit depends on the OperatoinalLimit.limitKind")

    #--------------------------------------------------------------------------
    #  Begin "VoltageLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "name", "aliasName", "value",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet", "OperationalLimitType",
                label="References"),
            dock="tab"),
        id="UCTE.OperationalLimits.VoltageLimit",
        title="VoltageLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "VoltageLimit" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" An extension to the Core Package that in association with the Terminal class models Connectivity, that is the physical definition of how equipment is connected together. In addition it models Topology, that is the logical definition of how equipment is connected via closed switches. The Topology definition is independent of the other electrical characteristics.An extension to the Core Package that in association with the Terminal class models Connectivity, that is the physical definition of how equipment is connected together. In addition it models Topology, that is the logical definition of how equipment is connected via closed switches. The Topology definition is independent of the other electrical characteristics.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from UCTE.Core import IdentifiedObject
from UCTE.Domain import ApparentPower



from enthought.traits.api import Instance, List, Property, Bool, Float
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#  "TopologicalNode" class:
#------------------------------------------------------------------------------

class TopologicalNode(IdentifiedObject):
    """ A set of connectivity nodes that, in the current network state, are connected together through any type of closed switches, including  jumpers. Topological nodes can change as the current network state changes (i.e., switches, breakers, etc. change state).A set of connectivity nodes that, in the current network state, are connected together through any type of closed switches, including  jumpers. Topological nodes can change as the current network state changes (i.e., switches, breakers, etc. change state).
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The control area into which the node is included.The control area into which the node is included.
    ControlArea = Instance("UCTE.ControlArea.ControlArea", allow_none=False,
        desc="The control area into which the node is included.The control area into which the node is included.",
        transient=True,
        opposite="TopologicalNode",
        editor=InstanceEditor(name="_controlareas"))

    def _get_controlareas(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.ControlArea.ControlArea" ]
        else:
            return []

    _controlareas = Property(fget=_get_controlareas)

    # The base voltage of the topologocial node.The base voltage of the TopologicalNode should match the BaseVoltage of the containing VoltageLevel if such a containing VoltageLevel is specified.The base voltage of the topologocial node.The base voltage of the TopologicalNode should match the BaseVoltage of the containing VoltageLevel if such a containing VoltageLevel is specified.
    BaseVoltage = Instance("UCTE.Core.BaseVoltage", allow_none=False,
        desc="The base voltage of the topologocial node.The base voltage of the TopologicalNode should match the BaseVoltage of the containing VoltageLevel if such a containing VoltageLevel is specified.The base voltage of the topologocial node.The base voltage of the TopologicalNode should match the BaseVoltage of the containing VoltageLevel if such a containing VoltageLevel is specified.",
        transient=True,
        opposite="TopologicalNode",
        editor=InstanceEditor(name="_basevoltages"))

    def _get_basevoltages(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Core.BaseVoltage" ]
        else:
            return []

    _basevoltages = Property(fget=_get_basevoltages)

    # The state voltage associated with the topological node.The state voltage associated with the topological node.
    SvVoltage = Instance("UCTE.StateVariables.SvVoltage",
        desc="The state voltage associated with the topological node.The state voltage associated with the topological node.",
        transient=True,
        opposite="TopologicalNode",
        editor=InstanceEditor(name="_svvoltages"))

    def _get_svvoltages(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.StateVariables.SvVoltage" ]
        else:
            return []

    _svvoltages = Property(fget=_get_svvoltages)

    # A topological node belongs to a topological islandA topological node belongs to a topological island
    TopologicalIsland = Instance("UCTE.Topology.TopologicalIsland", allow_none=False,
        desc="A topological node belongs to a topological islandA topological node belongs to a topological island",
        transient=True,
        opposite="TopologicalNodes",
        editor=InstanceEditor(name="_topologicalislands"))

    def _get_topologicalislands(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Topology.TopologicalIsland" ]
        else:
            return []

    _topologicalislands = Property(fget=_get_topologicalislands)

    # The island for which the node is an angle reference.   Normally there is one angle reference node for each island.The island for which the node is an angle reference.   Normally there is one angle reference node for each island.
    AngleRef_TopologicalIsland = Instance("UCTE.Topology.TopologicalIsland",
        desc="The island for which the node is an angle reference.   Normally there is one angle reference node for each island.The island for which the node is an angle reference.   Normally there is one angle reference node for each island.",
        transient=True,
        opposite="AngleRef_TopologicalNode",
        editor=InstanceEditor(name="_topologicalislands"))

    def _get_topologicalislands(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Topology.TopologicalIsland" ]
        else:
            return []

    _topologicalislands = Property(fget=_get_topologicalislands)

    # The connectivity node container to which the toplogical node belongs.The TopologicalNode will normally belong only to a VoltageLevel instance within a Substation.   All instances of TopologicalNode that are not X-nodes will require an association to a containing VoltageLevel instance.  The BaseVoltage of the VoltageLevel should match that of the TopologicalNode itself. A TopologicalNode object used for an X-node will not be contained, thus this association is specified as optional in the profile.The connectivity node container to which the toplogical node belongs.The TopologicalNode will normally belong only to a VoltageLevel instance within a Substation.   All instances of TopologicalNode that are not X-nodes will require an association to a containing VoltageLevel instance.  The BaseVoltage of the VoltageLevel should match that of the TopologicalNode itself. A TopologicalNode object used for an X-node will not be contained, thus this association is specified as optional in the profile.
    ConnectivityNodeContainer = Instance("UCTE.Core.ConnectivityNodeContainer",
        desc="The connectivity node container to which the toplogical node belongs.The TopologicalNode will normally belong only to a VoltageLevel instance within a Substation.   All instances of TopologicalNode that are not X-nodes will require an association to a containing VoltageLevel instance.  The BaseVoltage of the VoltageLevel should match that of the TopologicalNode itself. A TopologicalNode object used for an X-node will not be contained, thus this association is specified as optional in the profile.The connectivity node container to which the toplogical node belongs.The TopologicalNode will normally belong only to a VoltageLevel instance within a Substation.   All instances of TopologicalNode that are not X-nodes will require an association to a containing VoltageLevel instance.  The BaseVoltage of the VoltageLevel should match that of the TopologicalNode itself. A TopologicalNode object used for an X-node will not be contained, thus this association is specified as optional in the profile.",
        transient=True,
        opposite="TopologicalNode",
        editor=InstanceEditor(name="_connectivitynodecontainers"))

    def _get_connectivitynodecontainers(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Core.ConnectivityNodeContainer" ]
        else:
            return []

    _connectivitynodecontainers = Property(fget=_get_connectivitynodecontainers)

    # The terminals associated with the topological node.   This can be used as an alternative to the connectivity node path to terminal, thus making it unneccesary to model connedtivity nodes in some cases.   Note that the if connectivity nodes are in the model, this association would proably not be used.The terminals associated with the topological node.   This can be used as an alternative to the connectivity node path to terminal, thus making it unneccesary to model connedtivity nodes in some cases.   Note that the if connectivity nodes are in the model, this association would proably not be used.
    Terminal = List(Instance("UCTE.Core.Terminal"),
        desc="The terminals associated with the topological node.   This can be used as an alternative to the connectivity node path to terminal, thus making it unneccesary to model connedtivity nodes in some cases.   Note that the if connectivity nodes are in the model, this association would proably not be used.The terminals associated with the topological node.   This can be used as an alternative to the connectivity node path to terminal, thus making it unneccesary to model connedtivity nodes in some cases.   Note that the if connectivity nodes are in the model, this association would proably not be used.")

    # The short circuit apparent power drawn at this node when faulted.This is for Short Circuit only.The short circuit apparent power drawn at this node when faulted.This is for Short Circuit only.
    sShortCircuit = ApparentPower(desc="The short circuit apparent power drawn at this node when faulted.This is for Short Circuit only.The short circuit apparent power drawn at this node when faulted.This is for Short Circuit only.")

    # The topological node is equivalent and not real equipment.If this is missing, it is assumed to be False.  If it is an X-Node, this equivalent is required.The topological node is equivalent and not real equipment.If this is missing, it is assumed to be False.  If it is an X-Node, this equivalent is required.
    equivalent = Bool(desc="The topological node is equivalent and not real equipment.If this is missing, it is assumed to be False.  If it is an X-Node, this equivalent is required.The topological node is equivalent and not real equipment.If this is missing, it is assumed to be False.  If it is an X-Node, this equivalent is required.")

    # The ratio of zero sequence reactance per positive sequence reactance.This is for Short Circuit only.The ratio of zero sequence reactance per positive sequence reactance.This is for Short Circuit only.
    x0PerX = Float(desc="The ratio of zero sequence reactance per positive sequence reactance.This is for Short Circuit only.The ratio of zero sequence reactance per positive sequence reactance.This is for Short Circuit only.")

    # The ratio of zero sequence resistance to positive sequence resistance.This is for Short Circuit only.The ratio of zero sequence resistance to positive sequence resistance.This is for Short Circuit only.
    r0PerR = Float(desc="The ratio of zero sequence resistance to positive sequence resistance.This is for Short Circuit only.The ratio of zero sequence resistance to positive sequence resistance.This is for Short Circuit only.")

    # Ratio of positive sequence reactance per postive sequence resistance.This is for Short Circuit only.Ratio of positive sequence reactance per postive sequence resistance.This is for Short Circuit only.
    xPerR = Float(desc="Ratio of positive sequence reactance per postive sequence resistance.This is for Short Circuit only.Ratio of positive sequence reactance per postive sequence resistance.This is for Short Circuit only.")

    #--------------------------------------------------------------------------
    #  Begin "TopologicalNode" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "name", "aliasName", "sShortCircuit", "equivalent", "x0PerX", "r0PerR", "xPerR",
                label="Attributes"),
            VGroup("Model", "ControlArea", "BaseVoltage", "SvVoltage", "TopologicalIsland", "AngleRef_TopologicalIsland", "ConnectivityNodeContainer", "Terminal",
                label="References"),
            dock="tab"),
        id="UCTE.Topology.TopologicalNode",
        title="TopologicalNode",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TopologicalNode" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TopologicalIsland" class:
#------------------------------------------------------------------------------

class TopologicalIsland(IdentifiedObject):
    """ An electrically connected subset of the network. Topological islands can change as the current network state changes (i.e., disconnect switches, breakers, etc. change state).An electrically connected subset of the network. Topological islands can change as the current network state changes (i.e., disconnect switches, breakers, etc. change state).
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A topological node belongs to a topological islandA topological node belongs to a topological island
    TopologicalNodes = List(Instance("UCTE.Topology.TopologicalNode"),
        desc="A topological node belongs to a topological islandA topological node belongs to a topological island")

    # The angle reference for the island.   Normally there is one TopologicalNode that is selected as the angle reference for each island.   Other reference schemes exist, so the association is optional.The angle reference for the island.   Normally there is one TopologicalNode that is selected as the angle reference for each island.   Other reference schemes exist, so the association is optional.
    AngleRef_TopologicalNode = Instance("UCTE.Topology.TopologicalNode",
        desc="The angle reference for the island.   Normally there is one TopologicalNode that is selected as the angle reference for each island.   Other reference schemes exist, so the association is optional.The angle reference for the island.   Normally there is one TopologicalNode that is selected as the angle reference for each island.   Other reference schemes exist, so the association is optional.",
        transient=True,
        opposite="AngleRef_TopologicalIsland",
        editor=InstanceEditor(name="_topologicalnodes"))

    def _get_topologicalnodes(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Topology.TopologicalNode" ]
        else:
            return []

    _topologicalnodes = Property(fget=_get_topologicalnodes)

    #--------------------------------------------------------------------------
    #  Begin "TopologicalIsland" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "name", "aliasName",
                label="Attributes"),
            VGroup("Model", "TopologicalNodes", "AngleRef_TopologicalNode",
                label="References"),
            dock="tab"),
        id="UCTE.Topology.TopologicalIsland",
        title="TopologicalIsland",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TopologicalIsland" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" An extension to the Core Package that in association with the Terminal class models Connectivity, that is the physical definition of how equipment is connected together. In addition it models Topology, that is the logical definition of how equipment is connected via closed switches. The Topology definition is independent of the other electrical characteristics.An extension to the Core Package that in association with the Terminal class models Connectivity, that is the physical definition of how equipment is connected together. In addition it models Topology, that is the logical definition of how equipment is connected via closed switches. The Topology definition is independent of the other electrical characteristics.
"""

from ucte.core import IdentifiedObject

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim"

ns_uri = "http://iec.ch/TC57/2009/CIM-schema-cim14#Package_Topology"

class TopologicalNode(IdentifiedObject):
    """ A set of connectivity nodes that, in the current network state, are connected together through any type of closed switches, including  jumpers. Topological nodes can change as the current network state changes (i.e., switches, breakers, etc. change state).A set of connectivity nodes that, in the current network state, are connected together through any type of closed switches, including  jumpers. Topological nodes can change as the current network state changes (i.e., switches, breakers, etc. change state).
    """
    # <<< topological_node
    # @generated
    def __init__(self, s_short_circuit=0.0, equivalent=False, x0_per_x=0.0, r0_per_r=0.0, x_per_r=0.0, control_area=None, base_voltage=None, sv_voltage=None, topological_island=None, angle_ref_topological_island=None, connectivity_node_container=None, terminal=None, **kw_args):
        """ Initialises a new 'TopologicalNode' instance.
        """
        # The short circuit apparent power drawn at this node when faulted.This is for Short Circuit only.The short circuit apparent power drawn at this node when faulted.This is for Short Circuit only. 
        self.s_short_circuit = s_short_circuit

        # The topological node is equivalent and not real equipment.If this is missing, it is assumed to be False.  If it is an X-Node, this equivalent is required.The topological node is equivalent and not real equipment.If this is missing, it is assumed to be False.  If it is an X-Node, this equivalent is required. 
        self.equivalent = equivalent

        # The ratio of zero sequence reactance per positive sequence reactance.This is for Short Circuit only.The ratio of zero sequence reactance per positive sequence reactance.This is for Short Circuit only. 
        self.x0_per_x = x0_per_x

        # The ratio of zero sequence resistance to positive sequence resistance.This is for Short Circuit only.The ratio of zero sequence resistance to positive sequence resistance.This is for Short Circuit only. 
        self.r0_per_r = r0_per_r

        # Ratio of positive sequence reactance per postive sequence resistance.This is for Short Circuit only.Ratio of positive sequence reactance per postive sequence resistance.This is for Short Circuit only. 
        self.x_per_r = x_per_r


        self._control_area = None
        self.control_area = control_area

        self._base_voltage = None
        self.base_voltage = base_voltage

        self._sv_voltage = None
        self.sv_voltage = sv_voltage

        self._topological_island = None
        self.topological_island = topological_island

        self._angle_ref_topological_island = None
        self.angle_ref_topological_island = angle_ref_topological_island

        self._connectivity_node_container = None
        self.connectivity_node_container = connectivity_node_container

        self._terminal = []
        if terminal is not None:
            self.terminal = terminal
        else:
            self.terminal = []


        super(TopologicalNode, self).__init__(**kw_args)
    # >>> topological_node

    # <<< control_area
    # @generated
    def get_control_area(self):
        """ The control area into which the node is included.The control area into which the node is included.
        """
        return self._control_area

    def set_control_area(self, value):
        if self._control_area is not None:
            filtered = [x for x in self.control_area.topological_node if x != self]
            self._control_area._topological_node = filtered

        self._control_area = value
        if self._control_area is not None:
            self._control_area._topological_node.append(self)

    control_area = property(get_control_area, set_control_area)
    # >>> control_area

    # <<< base_voltage
    # @generated
    def get_base_voltage(self):
        """ The base voltage of the topologocial node.The base voltage of the TopologicalNode should match the BaseVoltage of the containing VoltageLevel if such a containing VoltageLevel is specified.The base voltage of the topologocial node.The base voltage of the TopologicalNode should match the BaseVoltage of the containing VoltageLevel if such a containing VoltageLevel is specified.
        """
        return self._base_voltage

    def set_base_voltage(self, value):
        if self._base_voltage is not None:
            filtered = [x for x in self.base_voltage.topological_node if x != self]
            self._base_voltage._topological_node = filtered

        self._base_voltage = value
        if self._base_voltage is not None:
            self._base_voltage._topological_node.append(self)

    base_voltage = property(get_base_voltage, set_base_voltage)
    # >>> base_voltage

    # <<< sv_voltage
    # @generated
    def get_sv_voltage(self):
        """ The state voltage associated with the topological node.The state voltage associated with the topological node.
        """
        return self._sv_voltage

    def set_sv_voltage(self, value):
        if self._sv_voltage is not None:
            self._sv_voltage._topological_node = None

        self._sv_voltage = value
        if self._sv_voltage is not None:
            self._sv_voltage._topological_node = self

    sv_voltage = property(get_sv_voltage, set_sv_voltage)
    # >>> sv_voltage

    # <<< topological_island
    # @generated
    def get_topological_island(self):
        """ A topological node belongs to a topological islandA topological node belongs to a topological island
        """
        return self._topological_island

    def set_topological_island(self, value):
        if self._topological_island is not None:
            filtered = [x for x in self.topological_island.topological_nodes if x != self]
            self._topological_island._topological_nodes = filtered

        self._topological_island = value
        if self._topological_island is not None:
            self._topological_island._topological_nodes.append(self)

    topological_island = property(get_topological_island, set_topological_island)
    # >>> topological_island

    # <<< angle_ref_topological_island
    # @generated
    def get_angle_ref_topological_island(self):
        """ The island for which the node is an angle reference.   Normally there is one angle reference node for each island.The island for which the node is an angle reference.   Normally there is one angle reference node for each island.
        """
        return self._angle_ref_topological_island

    def set_angle_ref_topological_island(self, value):
        if self._angle_ref_topological_island is not None:
            self._angle_ref_topological_island._angle_ref_topological_node = None

        self._angle_ref_topological_island = value
        if self._angle_ref_topological_island is not None:
            self._angle_ref_topological_island._angle_ref_topological_node = self

    angle_ref_topological_island = property(get_angle_ref_topological_island, set_angle_ref_topological_island)
    # >>> angle_ref_topological_island

    # <<< connectivity_node_container
    # @generated
    def get_connectivity_node_container(self):
        """ The connectivity node container to which the toplogical node belongs.The TopologicalNode will normally belong only to a VoltageLevel instance within a Substation.   All instances of TopologicalNode that are not X-nodes will require an association to a containing VoltageLevel instance.  The BaseVoltage of the VoltageLevel should match that of the TopologicalNode itself. A TopologicalNode object used for an X-node will not be contained, thus this association is specified as optional in the profile.The connectivity node container to which the toplogical node belongs.The TopologicalNode will normally belong only to a VoltageLevel instance within a Substation.   All instances of TopologicalNode that are not X-nodes will require an association to a containing VoltageLevel instance.  The BaseVoltage of the VoltageLevel should match that of the TopologicalNode itself. A TopologicalNode object used for an X-node will not be contained, thus this association is specified as optional in the profile.
        """
        return self._connectivity_node_container

    def set_connectivity_node_container(self, value):
        if self._connectivity_node_container is not None:
            filtered = [x for x in self.connectivity_node_container.topological_node if x != self]
            self._connectivity_node_container._topological_node = filtered

        self._connectivity_node_container = value
        if self._connectivity_node_container is not None:
            self._connectivity_node_container._topological_node.append(self)

    connectivity_node_container = property(get_connectivity_node_container, set_connectivity_node_container)
    # >>> connectivity_node_container

    # <<< terminal
    # @generated
    def get_terminal(self):
        """ The terminals associated with the topological node.   This can be used as an alternative to the connectivity node path to terminal, thus making it unneccesary to model connedtivity nodes in some cases.   Note that the if connectivity nodes are in the model, this association would proably not be used.The terminals associated with the topological node.   This can be used as an alternative to the connectivity node path to terminal, thus making it unneccesary to model connedtivity nodes in some cases.   Note that the if connectivity nodes are in the model, this association would proably not be used.
        """
        return self._terminal

    def set_terminal(self, value):
        for x in self._terminal:
            x._topological_node = None
        for y in value:
            y._topological_node = self
        self._terminal = value

    terminal = property(get_terminal, set_terminal)

    def add_terminal(self, *terminal):
        for obj in terminal:
            obj._topological_node = self
            self._terminal.append(obj)

    def remove_terminal(self, *terminal):
        for obj in terminal:
            obj._topological_node = None
            self._terminal.remove(obj)
    # >>> terminal


    def __str__(self):
        """ Returns a string representation of the TopologicalNode.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< topological_node.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the TopologicalNode.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "TopologicalNode", self.uri)
        if format:
            indent += ' ' * depth

        if self.control_area is not None:
            s += '%s<%s:TopologicalNode.control_area rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.control_area.uri)
        if self.base_voltage is not None:
            s += '%s<%s:TopologicalNode.base_voltage rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.base_voltage.uri)
        if self.sv_voltage is not None:
            s += '%s<%s:TopologicalNode.sv_voltage rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.sv_voltage.uri)
        if self.topological_island is not None:
            s += '%s<%s:TopologicalNode.topological_island rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.topological_island.uri)
        if self.angle_ref_topological_island is not None:
            s += '%s<%s:TopologicalNode.angle_ref_topological_island rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.angle_ref_topological_island.uri)
        if self.connectivity_node_container is not None:
            s += '%s<%s:TopologicalNode.connectivity_node_container rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.connectivity_node_container.uri)
        for obj in self.terminal:
            s += '%s<%s:TopologicalNode.terminal rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:TopologicalNode.s_short_circuit>%s</%s:TopologicalNode.s_short_circuit>' % \
            (indent, ns_prefix, self.s_short_circuit, ns_prefix)
        s += '%s<%s:TopologicalNode.equivalent>%s</%s:TopologicalNode.equivalent>' % \
            (indent, ns_prefix, self.equivalent, ns_prefix)
        s += '%s<%s:TopologicalNode.x0_per_x>%s</%s:TopologicalNode.x0_per_x>' % \
            (indent, ns_prefix, self.x0_per_x, ns_prefix)
        s += '%s<%s:TopologicalNode.r0_per_r>%s</%s:TopologicalNode.r0_per_r>' % \
            (indent, ns_prefix, self.r0_per_r, ns_prefix)
        s += '%s<%s:TopologicalNode.x_per_r>%s</%s:TopologicalNode.x_per_r>' % \
            (indent, ns_prefix, self.x_per_r, ns_prefix)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "TopologicalNode")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> topological_node.serialize


class TopologicalIsland(IdentifiedObject):
    """ An electrically connected subset of the network. Topological islands can change as the current network state changes (i.e., disconnect switches, breakers, etc. change state).An electrically connected subset of the network. Topological islands can change as the current network state changes (i.e., disconnect switches, breakers, etc. change state).
    """
    # <<< topological_island
    # @generated
    def __init__(self, topological_nodes=None, angle_ref_topological_node=None, **kw_args):
        """ Initialises a new 'TopologicalIsland' instance.
        """

        self._topological_nodes = []
        if topological_nodes is not None:
            self.topological_nodes = topological_nodes
        else:
            self.topological_nodes = []

        self._angle_ref_topological_node = None
        self.angle_ref_topological_node = angle_ref_topological_node


        super(TopologicalIsland, self).__init__(**kw_args)
    # >>> topological_island

    # <<< topological_nodes
    # @generated
    def get_topological_nodes(self):
        """ A topological node belongs to a topological islandA topological node belongs to a topological island
        """
        return self._topological_nodes

    def set_topological_nodes(self, value):
        for x in self._topological_nodes:
            x._topological_island = None
        for y in value:
            y._topological_island = self
        self._topological_nodes = value

    topological_nodes = property(get_topological_nodes, set_topological_nodes)

    def add_topological_nodes(self, *topological_nodes):
        for obj in topological_nodes:
            obj._topological_island = self
            self._topological_nodes.append(obj)

    def remove_topological_nodes(self, *topological_nodes):
        for obj in topological_nodes:
            obj._topological_island = None
            self._topological_nodes.remove(obj)
    # >>> topological_nodes

    # <<< angle_ref_topological_node
    # @generated
    def get_angle_ref_topological_node(self):
        """ The angle reference for the island.   Normally there is one TopologicalNode that is selected as the angle reference for each island.   Other reference schemes exist, so the association is optional.The angle reference for the island.   Normally there is one TopologicalNode that is selected as the angle reference for each island.   Other reference schemes exist, so the association is optional.
        """
        return self._angle_ref_topological_node

    def set_angle_ref_topological_node(self, value):
        if self._angle_ref_topological_node is not None:
            self._angle_ref_topological_node._angle_ref_topological_island = None

        self._angle_ref_topological_node = value
        if self._angle_ref_topological_node is not None:
            self._angle_ref_topological_node._angle_ref_topological_island = self

    angle_ref_topological_node = property(get_angle_ref_topological_node, set_angle_ref_topological_node)
    # >>> angle_ref_topological_node


    def __str__(self):
        """ Returns a string representation of the TopologicalIsland.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< topological_island.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the TopologicalIsland.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "TopologicalIsland", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.topological_nodes:
            s += '%s<%s:TopologicalIsland.topological_nodes rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.angle_ref_topological_node is not None:
            s += '%s<%s:TopologicalIsland.angle_ref_topological_node rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.angle_ref_topological_node.uri)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "TopologicalIsland")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> topological_island.serialize


# <<< topology
# @generated
# >>> topology
